/**
 * Copyright 2019-2023 Britanicus <marcusbritanicus@gmail.com>
 *
 * This file is a part of DesQ Shell (https://gitlab.com/DesQ/Shell)
 * DesQ Shell is the main Shell UI of the DesQ Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

#pragma once

#include <QDir>
#include <QDebug>
#include <QWidget>
#include <QHBoxLayout>
#include <QToolButton>

#include <desqui/PanelPlugin.hpp>

#ifndef QUIntList
#define QUIntList    QList<uint>
#endif

namespace WQt {
    class ToplevelManager;
    class Toplevel;
    class DesQShell;
    class DesQOutput;
}

class TaskButton;

class TasksPluginWidget : public DesQ::Panel::PluginWidget {
    Q_OBJECT

    public:
        TasksPluginWidget( QWidget *parent = nullptr );
        ~TasksPluginWidget();

        void addTask( WQt::Toplevel *tl );
        void removeTask( WQt::Toplevel *tl );

    private:
        WQt::ToplevelManager *tlm;

        WQt::DesQShell *shell;
        WQt::DesQOutput *output;

        QHBoxLayout *tasksLayout;

        QList<QList<QUIntList> > tasks;
        QMap<uint32_t, WQt::Toplevel *> idToplevelMap;
        QMap<uint32_t, TaskButton *> idTaskMap;
};


class TrayPlugin : public QObject, public DesQ::Plugin::PanelInterface {
    Q_OBJECT

    Q_PLUGIN_METADATA( IID "org.DesQ.Plugin.Panel" );
    Q_INTERFACES( DesQ::Plugin::PanelInterface );

    public:
        /* Name of the plugin */
        QString name() override {
            return "System Tray";
        }

        /* Icon for the plugin */
        QIcon icon() override {
            return QIcon::fromTheme( "org.kde.plasma.systemtray" );
        }

        /* The plugin version */
        QString version() override {
            return "0.0.8";
        }

        /* The clock Widget */
        DesQ::Panel::PluginWidget *widget( QWidget *parent ) override {
            return new TasksPluginWidget( parent );
        }
};
